/*****************************************************************************/
/*	SSDT Copyright (C) 2003 Paolo Ardoino				     */
/*****************************************************************************/
/*									     */
/* This program is free software; you can redistribute it and/or modify	     */
/* it under the terms of the GNU General Public License as published by	     */
/* the Free Software Foundation; either version 2 of the License, or	     */
/* (at your option) any later version.					     */
/* This program is distributed in the hope that it will be useful,	     */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of	     */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the	     */
/* GNU General Public License for more details.				     */
/*									     */
/* You should have received a copy of the GNU General Public License	     */
/* along with this program; if not, write to the Free Software		     */
/* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */
/*****************************************************************************/
/*Thanks to Fusys*/

#include <netinet/ip_icmp.h>

#define ICMP_HDR 8
#define IP_HDR 20
#define MAX_ICMP 1024L


int sockfd;
struct sockaddr_in clisrc;


int ICMP_init(void)
{
  int spoof_opt = 1;

  if((sockfd = socket(AF_INET, SOCK_RAW, IPPROTO_RAW)) < 0 ) {
    fprintf(stderr, "Error: Cannot create raw socket.\n");
    return -1;
  }
  if(setsockopt(sockfd, IPPROTO_IP, IP_HDRINCL, &spoof_opt, sizeof(spoof_opt)) < 0 ) {
    fprintf(stderr, "Error: Cannot create IP header.\n");
    return -1;
  }
  return 0;
}


void ICMP_reset(void)
{
  close(sockfd);
  return;
}


long ICMP_sp_send(unsigned char *ciphdata, unsigned long ciphsz, u_long sp_ip, u_long dest_ip)
{
  struct spoof {
    struct ip ip;
    struct icmp icmp;
    u_char data[MAX_ICMP];
  } sp_pk;
  int iplen = sizeof(struct ip);
  int icmplen = sizeof(struct icmp);
  unsigned long pklen, sent;
  struct sockaddr_in dest;
  int destlen;

  if (ciphsz + IP_HDR + ICMP_HDR > MAX_IP) {
    fprintf(stderr, "Error: ICMP packet size exceeded.\n");
    return -1;
  }
  if (ICMP_init() == -1) {
    return -1;
  }
  destlen = sizeof(dest);
  bzero((char *) &dest, destlen);
  dest.sin_family = AF_INET;
  dest.sin_addr.s_addr = dest_ip;
  pklen = ciphsz + sizeof(struct ip) + sizeof(struct icmp);
  memset(&sp_pk, 0, pklen); 
  sp_pk.ip.ip_v = 4;
  sp_pk.ip.ip_hl = 5;
  sp_pk.ip.ip_len = htons(iplen + icmplen + ciphsz);
  sp_pk.ip.ip_ttl = 255;
  sp_pk.ip.ip_p = IPPROTO_ICMP;
  sp_pk.ip.ip_src.s_addr = sp_ip;
  sp_pk.ip.ip_dst.s_addr = dest_ip;
  sp_pk.icmp.icmp_type = ICMP_ECHOREPLY;
  bzero((unsigned char *)sp_pk.icmp.icmp_data, MAX_ICMP);
  bcopy(ciphdata, sp_pk.icmp.icmp_data, ciphsz);
  sp_pk.icmp.icmp_cksum = in_chksum((u_short *) &sp_pk.icmp,(sizeof(struct icmp) + ciphsz));  
  if((sent = sendto(sockfd, &sp_pk, pklen, 0, (struct sockaddr *)&dest, destlen)) < 0 ) {
     fprintf(stderr ,"Error: ICMP sendto().\n");
     return -1;
  }
  ICMP_reset();
  if (sent != pklen) {
    fprintf(stderr, "Error: Wrong packet size returned by ICMP sendto().\n");
    return -1;
  }
  return sent;
}


int ICMP_recv_init(void)
{
  if((sockfd = socket(AF_INET, SOCK_RAW, IPPROTO_ICMP)) < 0 ) {
    fprintf(stderr, "Error: Cannot create raw ICMP socket.\n");
    return -1;
  }
  return 0;
}


long ICMP_recv(char *ciphdata, unsigned long ciphsz)
{
  struct recv {
    struct ip ip;
    struct icmp	icmp;
    char data[MAX_ICMP];
  } rcv_pk;
  unsigned long pklen, rcsz;
  int clilen = sizeof(clisrc);
  
  while(1) {
    pklen = ciphsz + sizeof(struct ip) + sizeof(struct icmp);
    memset(&rcv_pk, 0, pklen);
    if((rcsz = recvfrom(sockfd, &rcv_pk, pklen, 0, (struct sockaddr *) &clisrc, &clilen)) < 0)
      continue;
    break;
  }
  bcopy(rcv_pk.icmp.icmp_data, ciphdata, rcsz);  
  return rcsz;
}
