/*****************************************************************************/
/*	SSDT Copyright (C) 2003 Paolo Ardoino				     */
/*****************************************************************************/
/*									     */
/* This program is free software; you can redistribute it and/or modify	     */
/* it under the terms of the GNU General Public License as published by	     */
/* the Free Software Foundation; either version 2 of the License, or	     */
/* (at your option) any later version.					     */
/* This program is distributed in the hope that it will be useful,	     */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of	     */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the	     */
/* GNU General Public License for more details.				     */
/*									     */
/* You should have received a copy of the GNU General Public License	     */
/* along with this program; if not, write to the Free Software		     */
/* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */
/*****************************************************************************/


#include "ssdt.h"


int main(int argc, char *argv[])
{
  unsigned char *ciphdata=NULL;
  unsigned long ciphsz=0;
  unsigned int port = 0;
  
  printf("SSFT [v%s] - Spoofed Secure Data Transfer.\n", VERSION);
  printf("coded by Paolo Ardoino <ardoino.gnu@disi.unige.it>\n");
  if (argc == 1) {
    help(*argv);
  } else if (argc == 5 && strcmp(*(argv + 1), "-g") == 0) {
    if (genkey(atoi(*(argv + 2)), *(argv + 3), *(argv + 4)) == -1) {
      fprintf(stderr, "Error: Failed to generate RSA keys.\n");
      exit(EXIT_FAILURE);
    } else {
      printf("Done.\n");
    }
  } else if (argc == 6 && strcmp(*(argv + 1), "-si") == 0) {
    ciphdata = (unsigned char *)malloc(MAX_ICMP * sizeof(unsigned char));
    memset(ciphdata, '\0', MAX_ICMP);
    if ((ciphsz = rsa_encrypt(*(argv + 4), MAX_ICMP, *(argv + 5), ciphdata)) == -1) {
      fprintf(stderr, "Error: Failed to encrypt data.\n");
      exit(EXIT_FAILURE);
    }
    printf("Size of RSA encrypted data: %ld bytes.\n", ciphsz);
    printf("Sending spoofed ICMP packet.\n");
    if (ICMP_sp_send(ciphdata, ciphsz, nameResolve(*(argv + 2)), nameResolve(*(argv + 3))) == -1) {
      fprintf(stderr, "Error: Failed sending ICMP packet.\n");
      exit(EXIT_FAILURE);
    }
    printf("Sent.\n");
    free(ciphdata);
  } else if (argc == 8 && strcmp(*(argv + 1), "-su") == 0) {
    ciphdata = (unsigned char *)malloc(MAX_UDP * sizeof(unsigned char));
    memset(ciphdata, '\0', MAX_UDP);
    if ((ciphsz = rsa_encrypt(*(argv + 6), MAX_UDP, *(argv + 7), ciphdata)) == -1) {
      fprintf(stderr, "Error: Failed to encrypt data.\n");
      exit(EXIT_FAILURE);
    }
    printf("Size of RSA encrypted data: %ld bytes.\n", ciphsz);
    if (*(argv + 6)) {
      port = atol(*(argv + 4));
      if (port <= 0 || port > 65535) {
        fprintf(stderr, "Errot: Wrong port.\n");
	exit(EXIT_FAILURE);
      }
    }
    printf("Sending spoofed UDP packet.\n");
    if (UDP_sp_send(ciphdata, ciphsz, nameResolve(*(argv + 2)), nameResolve(*(argv + 3)), atoi(*(argv + 4)), atoi(*(argv + 5))) == -1) {
      fprintf(stderr, "Error: Failed sending UDP packet.\n");
      exit(EXIT_FAILURE);
    }
    printf("Sent.\n");
    free(ciphdata);
  } else {
    help(*argv);
  }
  return 0;
}


void help(char *pname)
{
  printf("MAX IP PACKET SIZE = %ld\n", MAX_IP);
  printf("MAX ICMP PACKET SIZE = %ld\n", MAX_ICMP);
  printf("MAX UDP PACKET SIZE = %ld\n", MAX_UDP);
  printf("%s -g <keysize> <secretkey_file> <publickey_file>\n", pname);
  printf("\tgenerates a RSA key pair of <keysize> bits.\n");
  printf("\t<keysize> :size in bits of the RSA key.\n");
  printf("\t<secretkey_file> :file for RSA Private Key.\n");
  printf("\t<publickey_file> :file for RSA Public Key.\n");
  printf("%s <-si> <spoof_ip> <dest_ip> <publickey_file> <filetosend>\n", pname);
  printf("%s <-su> <spoof_ip> <dest_ip> <src_port> <dest_port> <publickey_file> <filetosend>\n", pname);
  printf("\tsends a spoofed ICMP <-si> or UDP <-su> packet containing an RSA encrypted file.\n");
  printf("\t<spoofed_ip> :ip to spoof.\n");
  printf("\t<dest_ip> :destination ip.\n");
  printf("\t<src_port> :source port.\n");
  printf("\t<dest_port> :destination port.\n");
  printf("\t<publickey_file>: file containing RSA public key.\n");
  printf("\t<filetosend> :file containing file to ecnrypt and send as ICMP packet.\n");
  return;
}
