/*****************************************************************************/
/*	ISSDT Copyright (C) 2003 Paolo Ardoino				     */
/*****************************************************************************/
/*									     */
/* This program is free software; you can redistribute it and/or modify	     */
/* it under the terms of the GNU General Public License as published by	     */
/* the Free Software Foundation; either version 2 of the License, or	     */
/* (at your option) any later version.					     */
/* This program is distributed in the hope that it will be useful,	     */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of	     */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the	     */
/* GNU General Public License for more details.				     */
/*									     */
/* You should have received a copy of the GNU General Public License	     */
/* along with this program; if not, write to the Free Software		     */
/* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */
/*****************************************************************************/


#include "ssdt.h"


#define FNAMESZ 15


int main(int argc, char *argv[])
{
  unsigned char *ciphdata=NULL, *plaindata=NULL;
  unsigned long int rcsz=0, plsz=0;
  FILE *fp=NULL;
  char tmfile[FNAMESZ];
  long port = 0;

  printf("SSFT [v%s] - Spoofed Secure Data Transfer Daemon.\n", VERSION);
  printf("coded by Paolo Ardoino <ardoino.gnu@disi.unige.it>\n");
  if (argc == 1) {
    help(*argv);
  } else if (argc == 3 && *(argv + 2) && strcmp(*(argv + 1), "-ki") == 0) {
    ciphdata = (unsigned char *)malloc(MAX_ICMP * sizeof(unsigned char));
    plaindata = (unsigned char *)malloc(MAX_ICMP * sizeof(unsigned char));
    if(ICMP_recv_init() == -1) {
      exit(EXIT_FAILURE);
    }
    printf("Waiting ICMP packets.\n");
    while(1) {
      memset(tmfile, '\0', FNAMESZ);
      memset(ciphdata,'\0', MAX_ICMP);
      memset(plaindata,'\0', MAX_ICMP);
      rcsz = ICMP_recv(ciphdata, MAX_ICMP);
      if ((rcsz < IP_HDR + ICMP_HDR)) {
        continue;
      }
      printf("Size of received data: %ld bytes.\n", rcsz);
      plsz = rsa_decrypt(*(argv + 2), ciphdata, rcsz, plaindata);
      snprintf(tmfile, FNAMESZ - 1, "%ld", time(NULL) + 1);
      fp = fopen(tmfile, "w");
      fwrite(plaindata, plsz, 1, fp);
      fclose(fp);
      printf("Received data saved in '%s' file.\n\n", tmfile);
    }
    ICMP_reset();
  } else if (argc == 4 && *(argv + 2) && strcmp(*(argv + 1), "-ku") == 0) {
    ciphdata = (unsigned char *)malloc(MAX_UDP * sizeof(unsigned char));
    plaindata = (unsigned char *)malloc(MAX_UDP * sizeof(unsigned char));
    port = atol(*(argv + 3));
    if (port <= 0 || port > 65535) {
      fprintf(stderr, "Errot: Wrong port.\n");
      exit(EXIT_FAILURE);
    }
    if (UDP_recv_init(atol(*(argv + 3))) == -1) {
      exit(EXIT_FAILURE);
    }
    printf("Waiting UDP datagrams on %ld port.\n", port);
    while(1) {
      memset(tmfile, '\0', FNAMESZ);
      memset(ciphdata,'\0', MAX_UDP);
      memset(plaindata,'\0', MAX_UDP);
      rcsz = UDP_recv(ciphdata, MAX_UDP);
      if ((rcsz < IP_HDR + UDP_HDR)) {
        continue;
      }
      printf("Size of received data: %ld bytes.\n", rcsz);
      plsz = rsa_decrypt(*(argv + 2), ciphdata, rcsz, plaindata);
      snprintf(tmfile, FNAMESZ - 1, "%ld", time(NULL) + 1);
      fp = fopen(tmfile, "w");
      fwrite(plaindata, plsz, 1, fp);
      fclose(fp);
      printf("Received data saved in '%s' file.\n\n", tmfile);
    }
    UDP_reset();
  } else {
    help(*argv);
  }
  return 0;
}

void help(char *pname) 
{
  printf("%s <-ki> <secretkey_file>\n", pname);
  printf("Receives and decrypts RSA encrypted files in ICMP packets.\n");
  printf("\t<secretkey_file> contains your private key to decrypt received data.\n");
  printf("%s <-ku> <secretkey_file> <local_port>\n", pname);
  printf("Receives and decrypts RSA encrypted files in UDP datagrams.\n");
  printf("\t<secretkey_file> contains your private key to decrypt received data.\n");
  printf("\t<local_port> port to bind.\n");
  return;
}
