/*****************************************************************************/
/*	SSDT Copyright (C) 2003 Paolo Ardoino				     */
/*****************************************************************************/
/*									     */
/* This program is free software; you can redistribute it and/or modify	     */
/* it under the terms of the GNU General Public License as published by	     */
/* the Free Software Foundation; either version 2 of the License, or	     */
/* (at your option) any later version.					     */
/* This program is distributed in the hope that it will be useful,	     */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of	     */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the	     */
/* GNU General Public License for more details.				     */
/*									     */
/* You should have received a copy of the GNU General Public License	     */
/* along with this program; if not, write to the Free Software		     */
/* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */
/*****************************************************************************/

#include <netinet/udp.h>

#define UDP_HDR 8
#define MAX_UDP 1024L


int sockfd;
struct sockaddr_in serv;


int UDP_init(void)
{
  int spoof_opt = 1;

  if((sockfd = socket(AF_INET, SOCK_RAW, IPPROTO_RAW)) < 0 ) {
    fprintf(stderr, "Error: Cannot create raw socket.\n");
    return -1;
  }
  if(setsockopt(sockfd, IPPROTO_IP, IP_HDRINCL, &spoof_opt, sizeof(spoof_opt)) < 0 ) {
    fprintf(stderr, "Error: Cannot create IP header.\n");
    return -1;
  }
  return 0;
}


void UDP_reset(void)
{
  close(sockfd);
  return;
}


long UDP_sp_send(unsigned char *ciphdata, unsigned long ciphsz, unsigned long sp_ip, unsigned long dest_ip, unsigned long sport, unsigned long dport)
{
  struct spoof{
    struct iphdr ip;
    struct udphdr udp;
    char data[MAX_UDP];
  } sp_pk;
  struct sockaddr_in host_sockaddr;
  unsigned long pklen, sent;

  if (ciphsz + IP_HDR + ICMP_HDR > MAX_IP) {
    fprintf(stderr, "Error: UDP packet size exceeded.\n");
    return -1;
  }
  if (UDP_init() == -1) {
    return -1;
  }
  host_sockaddr.sin_addr.s_addr = dest_ip;
  host_sockaddr.sin_family = AF_INET;
  pklen = sizeof(struct iphdr) + sizeof(struct udphdr) + ciphsz + UDP_HDR;
  sp_pk.ip.ihl = 5;
  sp_pk.ip.version = 4;
  sp_pk.ip.tos = 0;
  sp_pk.ip.tot_len = htons(pklen);
  sp_pk.ip.id = 0;
  sp_pk.ip.frag_off = 0;
  sp_pk.ip.ttl = 64 ;
  sp_pk.ip.protocol = IPPROTO_UDP;
  sp_pk.ip.check = 0;
  sp_pk.ip.saddr = sp_ip;
  sp_pk.ip.daddr = host_sockaddr.sin_addr.s_addr;
  sp_pk.ip.check = in_chksum((u_short *)&sp_pk.ip, sizeof(struct iphdr));
  sp_pk.udp.source = htons(sport);
  sp_pk.udp.dest = htons(dport);
  sp_pk.udp.len = htons(pklen - sizeof(struct iphdr));
  sp_pk.udp.check = 0;
  bcopy(ciphdata, sp_pk.data, ciphsz);
  if ((sent = sendto(sockfd, &sp_pk, pklen, 0, (struct sockaddr*)&host_sockaddr, sizeof(host_sockaddr))) == -1) {
    fprintf(stderr ,"Error: UDP sendto().\n");
    return -1;
  }
  UDP_reset();
  return sent;
}


int UDP_recv_init(unsigned long port)
{
  if((sockfd = socket(AF_INET, SOCK_DGRAM, IPPROTO_UDP)) < 0 ) {
    fprintf(stderr, "Error: Cannot create UDP socket.\n");
    return -1;
  }
  bzero((char *)&serv, sizeof(serv));
  serv.sin_family = AF_INET;
  serv.sin_addr.s_addr = htonl(INADDR_ANY);
  serv.sin_port = htons(port);
  if (bind(sockfd, (struct sockaddr *)&serv, sizeof(serv)) != 0) {
    fprintf(stderr, "Error: bind().\n");
    return -1;
  }
  return 0;
}


long UDP_recv(char *ciphdata, unsigned long ciphsz)
{
  unsigned long pklen, rcsz;
  int servlen = sizeof(serv);
  
  while(1) {
    pklen = ciphsz + sizeof(struct iphdr) + sizeof(struct udphdr);
    if((rcsz = recvfrom(sockfd, ciphdata, pklen, 0, (struct sockaddr *) &serv, &servlen)) < 0)
      continue;
    break;
  }
  return rcsz;
}

